/*
 * Copyright (C) 2002, 2003, 2005 Philip Blundell <philb@gnu.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 */

#include <sys/types.h>
#include <stdlib.h>
#include <time.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>
#include <libintl.h>

#include <gtk/gtk.h>
#include <gdk-pixbuf/gdk-pixbuf.h>

#include <dbus/dbus.h>
#include <dbus/dbus-glib.h>

#include <bluetooth/bluetooth.h>

#include "pin-ui.h"

#define WRONG_ARGS_ERROR "org.bluez.Error.WrongArgs"

struct pin_request_context
{
  DBusConnection *connection;
  DBusMessage *message;
};

extern void bluez_pin_request (struct pin_request_context *ctx, gboolean outgoing, const gchar *address, const gchar *name);

static void
bluez_pin_send_dbus_reply (DBusConnection *connection, DBusMessage *message, const char *pin)
{
  if (pin)
    dbus_message_append_args(message,
			     DBUS_TYPE_STRING, &pin,
			     DBUS_TYPE_INVALID);

  dbus_connection_send (connection, message, NULL);

  dbus_message_unref (message);
}

void
dbus_pin_result (BluetoothPinRequest *req, gchar *result, void *user_data)
{
  struct pin_request_context *ctx = user_data;

  bluez_pin_send_dbus_reply (ctx->connection, ctx->message, result);

  g_free (ctx);

  g_object_unref (req);
}

DBusHandlerResult
bluez_pin_handle_dbus_request (DBusConnection *connection, DBusMessage *message)
{
  DBusMessageIter iter;
  gboolean out;
  bdaddr_t bdaddr, sbdaddr;
  int type;
  char *address;
  DBusMessage *reply;
  struct pin_request_context *ctx;
  unsigned char *bytes;
  int nbytes;
  BluetoothPinRequest *req;
  DBusError error = { 0 };

  memset(&bdaddr, 0, 6);
  dbus_error_init(&error);
  dbus_message_get_args(message, &error,
			DBUS_TYPE_BOOLEAN, &out,
			DBUS_TYPE_ARRAY, DBUS_TYPE_BYTE, &bytes, &nbytes,
			DBUS_TYPE_INVALID);
  if (dbus_error_is_set(&error)) {
    reply = dbus_message_new_error(message, WRONG_ARGS_ERROR,
					 error.name);
    dbus_error_free(&error);
    goto error;
  }
  if (nbytes != sizeof(bdaddr)) {
    reply = dbus_message_new_error (message, WRONG_ARGS_ERROR,
					  "Byte array incorrect size");
    goto error;
  }

  memcpy (&bdaddr, bytes, sizeof (bdaddr));

  reply = dbus_message_new_method_return (message);
  if (!reply)
    return DBUS_HANDLER_RESULT_NEED_MEMORY;

  ctx = g_malloc (sizeof (*ctx));
  ctx->message = reply;
  ctx->connection = connection;

  baswap (&sbdaddr, &bdaddr);
  address = batostr (&sbdaddr);

  req = bluetooth_pin_request_new (out, address, NULL);

  g_signal_connect (G_OBJECT (req), "result", G_CALLBACK (dbus_pin_result), ctx);

  return DBUS_HANDLER_RESULT_HANDLED;

 error:
  dbus_connection_send (connection, reply, NULL);

  dbus_message_unref (reply);

  return DBUS_HANDLER_RESULT_HANDLED;
}
